@@ -1,9 +1,9 @@
 """Moon coverage module."""
 
-from .esa import JUICE_CReMA
+from .esa import ESA_CREMAS, JUICE_CREMAS
 from .maps import CALLISTO, EARTH, EUROPA, GANYMEDE, IO, MOON, VENUS
 from .rois import ROI, CallistoROIs, GanymedeROIs, GeoJsonROI
-from .spice import SpicePool, SpiceRef, et, tdb, utc
+from .spice import MetaKernel, SpicePool, SpiceRef, datetime, et, tdb, utc
 from .trajectory import TourConfig, Trajectory
 from .version import __version__
 
@@ -17,12 +17,15 @@ __all__ = [
     'IO',
     'VENUS',
     'ROI',
-    'JUICE_CReMA',
+    'ESA_CREMAS',
+    'JUICE_CREMAS',
     'GeoJsonROI',
     'GanymedeROIs',
     'CallistoROIs',
+    'MetaKernel',
     'SpicePool',
     'SpiceRef',
+    'datetime',
     'et',
     'tdb',
     'utc',
