@@ -10,7 +10,7 @@ from PIL import Image
 
 from ..projections.axes import ProjAxes
 from ..projections.equi import Equirectangular
-from ..projections.ticks import lat_ticks, lon_e_ticks
+from ..ticks import lat_ticks, lon_e_ticks
 
 
 # ByPass DecompressionBombWarning for large images
