@@ -1,7 +1,7 @@
 """Miscellaneous module."""
 
 from .cache import cached_property, debug_cache
-from .depreciation import depreciated_renamed
+from .depreciation import DeprecationHelper, depreciated_renamed, warn
 from .download import debug_download, wget
 from .list import rindex
 from .logger import logger
@@ -15,6 +15,8 @@ __all__ = [
     'wget',
     'debug_download',
     'cached_property',
+    'warn',
+    'DeprecationHelper',
     'depreciated_renamed',
     'debug_cache',
 ]
