@@ -5,12 +5,14 @@ import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.axes import Axes
 from matplotlib.axis import XAxis, YAxis
+from matplotlib.cm import ScalarMappable
 from matplotlib.collections import LineCollection, PatchCollection
+from matplotlib.colors import Normalize
 from matplotlib.legend_handler import HandlerPolyCollection
 from matplotlib.patches import PathPatch
 from matplotlib.ticker import FixedLocator, NullLocator
 
-from .ticks import (
+from ..ticks import (
     deg_ticks, hr_ticks, km_s_ticks, km_ticks, lat_ticks, lon_e_ticks
 )
 
@@ -37,6 +39,7 @@ class ProjAxes(Axes):
         self.bg = bg
         self.bg_extent = bg_extent
         self.target = target
+        self._cbar = None
 
         super().__init__(*args, **kwargs)
 
@@ -187,18 +190,15 @@ class ProjAxes(Axes):
         extend = 'both' if cmin and cmax else 'min' \
             if cmin else 'max' if cmax else 'neither'
 
-        cbar = plt.colorbar(
-            lines, ax=self,
-            extend=extend,
-            shrink=.6,
-            aspect=40,
-            pad=0.05,
-            orientation=orientation,
-            format=fmt,
-            label=label,
-        )
+        cbar_kwargs = {
+            'cmap': cmap,
+            'orientation': orientation,
+            'extend': extend,
+            'format': fmt,
+            'label': label,
+        }
 
-        return cbar
+        return self.colorbar(vmin, vmax, **cbar_kwargs)
 
     def add_path(self, path, *args, **kwargs):
         """Draw path."""
@@ -224,6 +224,109 @@ class ProjAxes(Axes):
 
         return super().legend(*args, **kwargs)
 
+    def colorbar(self, vmin, vmax, cmap='turbo_r',
+                 orientation='horizontal',
+                 shrink=.6,
+                 aspect=40,
+                 pad=0.05,
+                 **kwargs):
+        """Add a standalone colorbar on the axis.
+
+        Parameters
+        ----------
+        vmin: int or float, optional
+            Color scaling min value. If ``None`` is provided (default)
+            the data are scaled to the lowest (not-NaN) value.
+        vmax: int or float, optional
+            Color scaling max value. If ``None`` is provided (default)
+            the data are scaled to the lowest (not-NaN) value.
+        cmap: str, optional
+            Matplotlib colormap name (default: `turbo_r`)
+        orientation: str, optional
+            Colorbar orientation (default: `horizontal`).
+        label: str, optional
+            Colorbar label (shortcuts are availables).
+        **kwargs:
+            Keyword attributes for :py:class:`Colorbar`.
+
+        Returns
+        -------
+        matplotlib.colorbar.Colorbar
+            Output colorbar.
+
+        """
+        norm = Normalize(vmin, vmax)
+
+        # Shortcut to format the ticks of known units
+        if 'label' in kwargs and kwargs['label'] in PROPS:
+            kwargs.update(zip(
+                ('label', 'format'), PROPS[kwargs['label']]
+            ))
+
+        self._cbar = self.figure.colorbar(
+            ScalarMappable(norm=norm, cmap=cmap),
+            ax=self,
+            orientation=orientation,
+            shrink=shrink,
+            aspect=aspect,
+            pad=pad,
+            **kwargs
+        )
+
+        return self._cbar
+
+    def twin_colorbar(self, label=None,
+                      format=None, offset=.05):  # pylint: disable=redefined-builtin
+        """Twin colorbar with a secondary axis.
+
+        Parameters
+        ----------
+        label: str, optional
+            Twin colorbar label (no shortcut).
+        format: matplotlib.ticker.Formatter, optional
+            Optional ticks formatter.
+        offset: float, optional
+            Colorbar offset (default: `0.05`).
+
+        """
+        if self._cbar is None:
+            raise ValueError('No parent colorbar found.')
+
+        pos = self._cbar.ax.get_position()
+        self._cbar.ax.set_aspect('auto')  # change default `equal` to `auto`
+
+        # Shift the colorbar to avoid to overlap the figure
+        if self._cbar.orientation == 'horizontal':
+            pos.y0 -= offset
+            pos.y1 -= offset
+            ax = self._cbar.ax.twiny()
+            ax.set_xlim(self._cbar.ax.get_xlim())
+
+            if label:
+                ax.set_xlabel(label)
+            if format:
+                ax.xaxis.set_major_formatter(format)
+
+        else:
+            pos.x0 += offset
+            pos.x1 += offset
+            ax = self._cbar.ax.twinx()
+            ax.set_ylim(self._cbar.ax.get_ylim())
+
+            if label:
+                ax.set_ylabel(label)
+            if format:
+                ax.yaxis.set_major_formatter(format)
+
+        self._cbar.ax.set_position(pos)
+        ax.set_position(pos)
+
+        # Remove the frame borders
+        for border in ax.spines:
+            ax.spines[border].set_visible(False)
+
+        return ax
+
     def set_longitude_grid(self, degrees):
         """Set the number of degrees between each longitude grid."""
         grid = np.linspace(0, 360, int(360 / degrees) + 1).astype(int)
