@@ -2,9 +2,9 @@
 
 import numpy as np
 
-from matplotlib.path import Path
-from matplotlib.patches import PathPatch
 from matplotlib.collections import PatchCollection, PathCollection
+from matplotlib.patches import PathPatch
+from matplotlib.path import Path
 
 from ..math import cs
 
@@ -177,6 +177,7 @@ class Projection:
             linewidth=patch.get_linewidth(),
             linestyle=patch.get_linestyle(),
             zorder=patch.get_zorder(),
+            label=patch.get_label(),
         )
 
     def xy_collection(self, collection):
@@ -205,6 +206,7 @@ class Projection:
             linewidth=collection.get_linewidth(),
             linestyle=collection.get_linestyle(),
             zorder=collection.get_zorder(),
+            label=collection.get_label(),
         )
 
     def meridians(self, lons_e=None, exclude=None,
