@@ -42,6 +42,9 @@ class AbstractItem:
     def __eq__(self, other):
         return str(self) == other
 
+    def __getitem__(self, item):
+        return getattr(self, item)
+
 
 class AbstractCollection(UserDict):
     """Abstract collection."""
