@@ -4,14 +4,14 @@ from copy import copy
 
 import numpy as np
 
-from matplotlib.path import Path
-from matplotlib.patches import PathPatch
 from matplotlib.collections import PatchCollection
+from matplotlib.patches import PathPatch
+from matplotlib.path import Path
 
-from .abstract import AbstractItem, AbstractCollection
-from .categories import (Category, CategoriesCollection,
-                         SubCategory, SubCategoriesCollection)
-
+from .abstract import AbstractCollection, AbstractItem
+from .categories import (
+    CategoriesCollection, Category, SubCategoriesCollection, SubCategory
+)
 from ..projections import Equirectangular
 
 
@@ -157,17 +157,17 @@ class ROI(AbstractItem):
             if 'edgecolor' not in kwargs:
                 kwargs['edgecolor'] = self.get_ec()
 
-        if 'alpha' not in kwargs:
-            kwargs['alpha'] = self.get_alpha()
+        defaults = {
+            'alpha': self.get_alpha,
+            'linewidth': self.get_linewidth,
+            'linestyle': self.get_linestyle,
+            'zorder': self.get_zorder,
+            'label': self.get_label,
+        }
 
-        if 'linewidth' not in kwargs:
-            kwargs['linewidth'] = self.get_linewidth()
-
-        if 'linestyle' not in kwargs:
-            kwargs['linestyle'] = self.get_linestyle()
-
-        if 'zorder' not in kwargs:
-            kwargs['zorder'] = self.get_zorder()
+        for kwarg, func in defaults.items():
+            if kwarg not in kwargs:
+                kwargs[kwarg] = func()
 
         patch = PathPatch(self.path, **kwargs)
 
@@ -204,6 +204,10 @@ class ROI(AbstractItem):
         """ROI default zorder."""
         return getattr(self, 'zorder', 1)
 
+    def get_label(self):
+        """ROI default label."""
+        return getattr(self, 'label', '')
+
     def contains(self, pts):
         """Check if points are inside the pixel.
 
@@ -294,6 +298,9 @@ class ROIsCollection(AbstractCollection):
         if 'zorder' not in kwargs:
             kwargs['zorder'] = self.get_zorder()
 
+        if 'label' not in kwargs:
+            kwargs['label'] = self.get_label()
+
         coll = PatchCollection(patches, **kwargs)
 
         if hasattr(self, 'target'):
@@ -329,6 +336,10 @@ class ROIsCollection(AbstractCollection):
         """Default zorder."""
         return default
 
+    def get_label(self, default=''):  # pylint: disable=no-self-use
+        """Default label."""
+        return default
+
     def contains(self, pts):
         """Check if a list of points are in the ROIs collection."""
         return np.any([roi.contains(pts) for roi in self], axis=0)
