@@ -1,9 +1,13 @@
 """Spice kernel pool module."""
 
+from functools import wraps
+
 import numpy as np
 
 import spiceypy as sp
 
+from .kernel import get_item
+from .metakernel import MetaKernel
 from .references import SpiceRef
 from .times import tdb, utc
 from ..misc import depreciated_renamed, logger
@@ -16,6 +20,8 @@ class MetaSpicePool(type):
     """Meta Spice kernel pool object."""
     # pylint: disable=no-value-for-parameter, unsupported-membership-test
 
+    MK_HASH = {}
+
     def __repr__(cls):
         n = int(cls)
         if n == 0:
@@ -35,63 +41,89 @@ class MetaSpicePool(type):
         return cls.count()
 
     def __hash__(cls):
-        return cls.get_pool_hash()
+        return cls.hash(cls.kernels)
 
     def __eq__(cls, other):
-        if isinstance(other, str):
-            return cls == [other]
+        if isinstance(other, (str, tuple, list)):
+            return hash(cls) == cls.hash(other)
 
-        if isinstance(other, list):
-            return cls == tuple(other)
-
-        return hash(cls) == hash(other)
+        return hash(cls) == other
 
     def __iter__(cls):
         return iter(cls.kernels)
 
-    def __contains__(cls, el):
-        return cls.contains(el)
+    def __contains__(cls, kernel):
+        return cls.contains(kernel)
 
-    def __add__(cls, el):
-        return cls.add(el)
+    def __add__(cls, kernel):
+        return cls.add(kernel)
 
-    def __sub__(cls, el):
-        return cls.remove(el)
+    def __sub__(cls, kernel):
+        return cls.remove(kernel)
+
+    def __getitem__(cls, item):
+        return get_item(item)
 
     @staticmethod
     def count() -> int:
         """Count the number of kernels in the pool."""
         return int(sp.ktotal('ALL'))
 
-    def get_pool(cls):
-        """Get Spice kernel pool files."""
+    @property
+    def kernels(cls):
+        """Return the list of kernels loaded in the pool."""
         return tuple(
             sp.kdata(i, 'ALL')[0] for i in range(cls.count())
         )
 
-    def get_pool_hash(cls):
-        """Get Spice pool content hash."""
-        return hash(cls.get_pool())
+    def hash(cls, kernels) -> int:
+        """Hash a (meta)kernel or a list of (meta)kernels."""
+        if isinstance(kernels, (str, MetaKernel)):
+            return cls.hash((kernels, ))
 
-    @property
-    def kernels(cls):
-        """Return the lis of kernels loaded in the pool."""
-        return cls.get_pool()
+        kernels_hash = ()
+        for kernel in kernels:
+            if isinstance(kernel, MetaKernel):
+                mk = kernel
+                # Hash of the metakernel and all the `kernels` loaded with it
+                kernels_hash += (hash(mk), *(hash(k) for k in mk.kernels))
+
+            elif kernel in cls.MK_HASH:  # Check if the kernel is in mk hash cached
+                kernels_hash += (cls.MK_HASH[kernel],)
+
+            elif kernel is not None:  # If not found, use the kernel hash if not None
+                kernels_hash += (hash(kernel),)
+
+        return hash(kernels_hash)
 
     def contains(cls, kernel):
         """Check if the kernel is in the pool."""
-        return kernel in cls.kernels
+        return kernel in cls.kernels or hash(kernel) in cls.MK_HASH.values()
 
     def add(cls, kernel, purge=False):
         """Add a kernel to the pool."""
         if purge:
             cls.purge()
 
-        if kernel in cls:
+        if isinstance(kernel, (tuple, list)):
+            for _kernel in kernel:
+                cls.add(_kernel, purge=False)
+
+        elif kernel in cls:
             raise ValueError(f'Kernel `{kernel}` is already in the pool.')
 
-        log_spice_pool.debug('Add %s', kernel)
-        sp.furnsh(kernel)
+        elif kernel is not None:
+            log_spice_pool.debug('Add `%s` in the SPICE pool', kernel)
+
+            if isinstance(kernel, MetaKernel):
+                with kernel as mk:
+                    # `mk` is the name of a `NamedTemporaryFile` (see `MetaKernel`)
+                    sp.furnsh(mk)
+
+                    log_spice_pool.debug('Cache metakernel original hash.')
+                    cls.MK_HASH[mk] = hash(kernel)
+            else:
+                sp.furnsh(kernel)
 
     def remove(cls, kernel):
         """Remove the kernel from the pool if present."""
@@ -101,11 +133,11 @@ class MetaSpicePool(type):
         log_spice_pool.debug('Remove %s', kernel)
         sp.unload(kernel)
 
-    @staticmethod
-    def purge():
+    def purge(cls):
         """Purge the pool from all its content."""
-        log_spice_pool.info('Empty the pool')
+        log_spice_pool.info('Purge the pool')
         sp.kclear()
+        cls.MK_HASH = {}
 
     def windows(cls, *refs, fmt='UTC'):
         """Get kernels windows on a collection of bodies in the pool.
@@ -265,6 +297,29 @@ class SpicePool(metaclass=MetaSpicePool):
     """Spice kernel pool object."""
 
 
+def check_kernels(func):  # pylint: disable=no-self-use
+    """Spice Pool kernels checker decorator.
+
+    The parent object must implement a :py:func:`__hash__`
+    function and have a :py:attr:`kernels` attribute.
+
+    """
+    @wraps(func)
+    def wrapper(_self, *args, **kwargs):
+        """Check if the content of pool have changed.
+
+        If the content changed, the pool will be purge and the kernels reloaded.
+
+        """
+        if SpicePool != hash(_self):
+            log_spice_pool.info(
+                'The content of the pool changed -> the kernels will be reloaded.')
+            SpicePool.add(_self.kernels, purge=True)
+
+        return func(_self, *args, **kwargs)
+    return wrapper
+
+
 @depreciated_renamed
 class SPICEPool(SpicePool):
     """Spice kernel pool object."""
