@@ -5,8 +5,9 @@ import numpy as np
 import spiceypy as sp
 
 from .fov import SpiceFieldOfView
+from .kernel import get_item
 from .times import sclk
-from ..misc import cached_property, depreciated_renamed
+from ..misc import cached_property, depreciated_renamed, warn
 
 
 def spice_name_code(ref):
@@ -71,6 +72,9 @@ class AbstractSpiceRef:
     def __eq__(self, other):
         return str(self) == other or int(self) == other
 
+    def __getitem__(self, item):
+        return get_item(item)
+
     @property
     def code(self):
         """SPICE reference ID as string."""
@@ -98,6 +102,9 @@ class SpiceBody(AbstractSpiceRef):
 
     """
 
+    def __getitem__(self, item):
+        return get_item(f'BODY{int(self)}_{item.upper()}')
+
     def is_valid(self):
         """Check if the code is valid for a SPICE body.
 
@@ -141,7 +148,7 @@ class SpiceBody(AbstractSpiceRef):
     @cached_property
     def radii(self):
         """Body radii, if available (km)."""
-        return sp.bodvrd(str(self), 'RADII', 3)[1]
+        return self['RADII']
 
     @property
     def radius(self):
@@ -166,13 +173,13 @@ class SpiceBody(AbstractSpiceRef):
     @property
     def f(self):
         """Body flattening coefficient, if available (km)."""
-        re, _, rp = self.radii
+        re, _, rp = self.radii  # pylint: disable=unbalanced-tuple-unpacking
         return (re - rp) / re
 
     @cached_property
     def mu(self):
         """Gravitational parameter (GM, km³/sec²)."""
-        return sp.bodvrd(str(self), 'GM', 1)[1]
+        return self['GM']
 
 
 class SpiceObserver(AbstractSpiceRef):
@@ -302,6 +309,8 @@ class SpiceInstrument(SpiceObserver, SpiceFieldOfView):
         Instrument name or code id.
 
     """
+    def __getitem__(self, item):
+        return get_item(f'INS{int(self)}_{item.upper()}')
 
     def is_valid(self):
         """Check if the code is valid for a SPICE instrument.
@@ -344,8 +353,35 @@ class SpiceInstrument(SpiceObserver, SpiceFieldOfView):
 
     @cached_property
     def spacecraft(self):
-        """Parent spacecraft."""
-        return SpiceSpacecraft(-(-int(self) // 1_000))
+        """Parent spacecraft.
+
+        Warning
+        -------
+        The current definition of JUICE PEP instruments IDs
+        (in `juice_pep_v09.ti`) are out-of-range NAIF code rules.
+        This special case is expected to be an exception and manually
+        fixed here with a ``DepreciationWarning``.
+        See `issue #12`_ to get more details.
+
+        .. _`issue #12`: https://juigitlab.esac.esa.int/datalab/moon-coverage/-/issues/12
+
+        """
+        try:
+            return SpiceSpacecraft(-(-int(self) // 1_000))
+
+        except ValueError as err:
+            if str(self).startswith('JUICE_PEP_'):
+                warn.warning(
+                    'Invalid JUICE/PEP instrument NAIF IDs (%i) for `%s`. '
+                    'The parent spacecraft ID is manually set to `JUICE` (-28). '
+                    'See issue #12 '
+                    '(https://juigitlab.esac.esa.int/datalab/moon-coverage/-/issues/12) '
+                    'for more details.',
+                    int(self), str(self)
+                )
+                return SpiceSpacecraft('JUICE')
+
+            raise err
 
     def sclk(self, *time):
         """Continuous encoded parent spacecraft clock ticks.
