@@ -1,4 +1,10 @@
-"""SPICE times module."""
+"""SPICE times module.
+
+The full metakernel specifications are available on NAIF website:
+
+https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/req/time.html
+
+"""
 
 import re
 
@@ -12,18 +18,25 @@ def et(utc, *times):
 
     Parametes
     ---------
-    utc: str
+    utc: any
         Input UTC time. All SPICE time formats are accepted.
 
         For example:
-            - YYYY-MM-DDThh:mm:ss[.ms][Z]
-            - YYYY-MON-DD hh:mm:ss
+        - YYYY-MM-DDThh:mm:ss[.ms][Z]
+        - YYYY-MON-DD hh:mm:ss
+
+        Multiple iterable input (``slice``, ``list``,
+        ``tuples``, ``numpy.ndarray``) are accepted as well.
+
+        If an ``int`` or ``float`` is provided,
+        the value is considered to be in ET and is not
+        not converted.
 
     *times: str
         Addition input UTC time(s) to parse.
 
-    Note
-    ----
+    Caution
+    -------
     The value is rounded at 1e-3 precision to avoid
     conversion rounding error with datatime at `ms`
     precisions values.
@@ -32,11 +45,14 @@ def et(utc, *times):
     if times:
         return [et(t) for t in [utc, *times]]
 
-    if isinstance(utc, (np.int, np.float)):
+    if isinstance(utc, (int, float, np.int64, np.float64)):
         return utc
 
+    if isinstance(utc, slice):
+        return et_range(utc.start, utc.stop, utc.step)
+
     if isinstance(utc, (tuple, list, np.ndarray)):
-        return [et(t) for t in utc]
+        return np.hstack([et(t) for t in utc]) if any(utc) else np.array([])
 
     utc = utc[:-1] if str(utc).endswith('Z') else str(utc)
     return np.round(sp.str2et(utc), 3)
@@ -277,7 +293,7 @@ def et_ranges(*ranges):
     ])
 
 
-def et_ca_range(t, *dt):
+def et_ca_range(t, *dt, et_min=None, et_max=None):
     """Ephemeris times around closest approach with a redefined sequence.
 
     Parameters
@@ -311,6 +327,12 @@ def et_ca_range(t, *dt):
 
         = 2,041 points around the CA point.
 
+    et_min: float, optional
+        Smallest valid value of ET (default: `None`).
+
+    et_max: float, optional
+        Largest valid value of ET (default: `None`).
+
     Returns
     -------
     [float, …]
@@ -325,7 +347,8 @@ def et_ca_range(t, *dt):
         dt = [(10, 'm', '1 sec'), (1, 'h', '10 sec'),
               (2, 'h', '1 min'), (12, 'h', '10 min')]
 
-    t = np.datetime64(t)
+    if not isinstance(t, np.datetime64):
+        t = np.datetime64(t)
 
     starts = [t - np.timedelta64(v, u) for v, u, _ in dt[::-1]] + \
              [t + np.timedelta64(v, u) for v, u, _ in dt[:-1]]
@@ -337,4 +360,15 @@ def et_ca_range(t, *dt):
 
     ranges = np.transpose([starts, stops, steps])
 
-    return et_ranges(*ranges)
+    ets = et_ranges(*ranges)
+
+    if et_min:
+        if et_max:
+            return ets[(et_min <= ets) & (ets <= et_max)]
+
+        return ets[et_min <= ets]
+
+    if et_max:
+        return ets[ets <= et_max]
+
+    return ets
