@@ -96,7 +96,7 @@ def radec(vec):
     Parameters
     ----------
     vec: tuple
-        Input XYZ cartesian vector coordinates i J200 frame.
+        Input XYZ cartesian vector coordinates in J200 frame.
 
     Returns
     -------
@@ -121,6 +121,7 @@ def sub_obs_pt(time, observer, target, abcorr='NONE', method='NEAR POINT/ELLIPSO
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
+        It refers to time at the observer location.
     observer: str
         Observer name.
     target: str
@@ -197,11 +198,6 @@ def sc_state(time, spacecraft, target, abcorr='NONE'):
         If a list of time were provided, the results will be stored
         in a (6, N) array.
 
-    Warning
-    -------
-    If a aberration correction is applied, you must use the `reception` case
-    (departing from the target location at `ET - LT`).
-
     See Also
     --------
     spiceypy.spkezr
@@ -218,6 +214,7 @@ def sc_state(time, spacecraft, target, abcorr='NONE'):
 
     state, _ = sp.spkezr(str(target), time, target.frame,
                          abcorr, str(spacecraft))
+
     return np.negative(state)
 
 
@@ -228,6 +225,7 @@ def attitude(time, observer, ref='J2000'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
+        It refers to time at the observer location.
     observer: str or SpiceSpacecraft or SpiceInstrument
         Spacecraft or instrument name.
     ref: str, optional
@@ -304,6 +302,11 @@ def intersect_pt(time, observer, target, frame, ray, limb=False,
         If a list of time/ray were provided, the results will be stored
         in a (3, N) array.
 
+    Warning
+    -------
+    Currently the limb intersection parameter is only available for
+    ``abcorr='NONE'`` (an ``NotImplementedError`` will be raised).
+
     See Also
     --------
     spiceypy.sincpt
@@ -449,6 +452,7 @@ def local_time(time, lon, target, lon_type='PLANETOCENTRIC'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
+        It refers to time at the target surface point location.
     lon: float or list or tuple
         Longitude of surface point (degree).
     target: str
@@ -470,7 +474,6 @@ def local_time(time, lon, target, lon_type='PLANETOCENTRIC'):
         were provided, the results will be stored
         in an array.
 
-
     Raises
     ------
     ValueError
@@ -517,7 +520,7 @@ def illum_angles(time, observer, target, pt, abcorr='NONE', method='ELLIPSOID'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
-        It refers to time at the surface point.
+        It refers to time at the target surface point location.
     observer: str
         Observer name.
     target: str
@@ -596,7 +599,7 @@ def sun_pos(time, target, abcorr='NONE'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
-        It refers to time at the target's location.
+        It refers to time at the target's center location.
     target: str
         Target body name.
     abcorr: str, optional
@@ -640,7 +643,7 @@ def solar_longitude(time, target, abcorr='NONE'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
-        It refers to time at the target's location.
+        It refers to time at the target's center location.
     target: str
         Target body name.
     abcorr: str, optional
@@ -691,7 +694,7 @@ def true_anomaly(time, target, abcorr='NONE', frame='ECLIPJ2000'):
     ----------
     time: float or list or tuple
         Ephemeris Time or UTC time input(s).
-        It refers to time at the target's location.
+        It refers to time at the target's center location.
     target: str
         Target body name.
     abcorr: str, optional
@@ -744,12 +747,15 @@ def groundtrack_velocity(target, state):
     This formula is only valid for a spheroid elongated along the
     axis of rotation (``c``). It is not correct for a generic ellipsoid.
 
+    No aberration correction is applied.
+
     Parameters
     ----------
     target: str
         Target body name.
     state: str
         Target -> observer state position and velocity vectors.
+        Computed at the observer time.
 
     Returns
     -------
