@@ -1,6 +1,7 @@
 """Trajectory module."""
 
 from .config import TourConfig, TrajectoryConfig
+from .fovs import debug_fovs
 from .trajectory import (
     Flyby, InstrumentFlyby, InstrumentTrajectory, SpacecraftFlyby,
     SpacecraftTrajectory, Trajectory, debug_trajectory
@@ -16,5 +17,6 @@ __all__ = [
     'InstrumentTrajectory',
     'TourConfig',
     'debug_trajectory',
+    'debug_fovs',
     'TrajectoryConfig',  # Depreciated
 ]
