@@ -2,18 +2,17 @@
 
 import numpy as np
 
-from matplotlib.collections import PathCollection
-from matplotlib.path import Path
-
-from .mask_traj import MaskedTrajectory
+from .fovs import FovsCollection
+from .mask_traj import MaskedInstrumentTrajectory, MaskedSpacecraftTrajectory
+from ..events import EventsList, EventWindow
 from ..math.vectors import angle, ell_norm, norm
 from ..misc import cached_property, logger
 from ..spice import (
-    SpiceBody, SpiceInstrument, SpicePool, SpiceRef,
-    SpiceSpacecraft, et, et_ca_range, et_range, utc
+    SpiceAbCorr, SpiceBody, SpiceInstrument, SpicePool, SpiceRef,
+    SpiceSpacecraft, check_kernels, et, et_ca_range, utc
 )
 from ..spice.toolbox import (
-    attitude, boresight_pt, fov_pts, groundtrack_velocity,
+    attitude, boresight_pt, groundtrack_velocity,
     illum_angles, local_time, radec, rlonlat, sc_state,
     solar_longitude, sub_obs_pt, sun_pos, true_anomaly
 )
@@ -44,6 +43,9 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         the target's position and orientation.
         Only the SPICE keys are accepted.
 
+    exclude: EventWindow or EventsList
+        Event window or list of events to exclude from the analysis.
+
     Raises
     ------
     ValueError
@@ -53,18 +55,17 @@ class Trajectory:  # pylint: disable=too-many-public-methods
 
     TOO_MANY_POINTS_WARNING = 1_000_000
 
-    def __init__(self, kernels, observer, target, ets, abcorr='NONE'):
+    def __init__(self, kernels, observer, target, ets, abcorr='NONE', exclude=None):
         self.kernels = kernels
 
-        # Init SPICE references
-        self.target = SpiceBody(target)
+        # Init SPICE references and times
+        self.target = target
         self.observer = observer
-
-        # Init ephemeris times
+        self.exclude = exclude
         self.ets = ets
 
         # Optional parameters
-        self.abcorr = abcorr
+        self.abcorr = SpiceAbCorr(abcorr)
 
     def __repr__(self):
         return (
@@ -102,7 +103,9 @@ class Trajectory:  # pylint: disable=too-many-public-methods
                 observer,
                 self.target,
                 self.ets,
-                abcorr='NONE')
+                abcorr=self.abcorr,
+                exclude=self.exclude,
+            )
 
         except ValueError:
             pass
@@ -113,43 +116,46 @@ class Trajectory:  # pylint: disable=too-many-public-methods
                 self.observer,
                 SpiceBody(item),
                 self.ets,
-                abcorr='NONE')
+                abcorr=self.abcorr,
+                exclude=self.exclude
+            )
         except KeyError:
             pass
 
         raise KeyError(
-            'The item must be a spacecraft, an instrument, a target body or the SUN. ')
+            'The item must be a spacecraft, an instrument, a target body or the SUN.')
+
+    def __hash__(self):
+        """Kernels hash."""
+        return self._kernels_hash
+
+    @cached_property
+    def _kernels_hash(self):
+        """Expected Spice Pool kernels hash."""
+        return SpicePool.hash(self.kernels)
 
     @property
-    def kernels(self):
-        """Get the list of kernels to used."""
-        return self.__kernels
+    def target(self):
+        """Trajectory target."""
+        return self.__target
 
-    @kernels.setter
-    def kernels(self, kernels):
-        """List of kernels setter.
-
-        Note
-        ----
-        SPICE pool content is checked to see if
-        all the kernels are correctly loaded.
-        If it's not the case, the pool will
-        be purge and refilled.
-
-        """
-        self.__kernels = kernels
-
-        if SpicePool != kernels:
-            log_traj.info('Load the kernels to the pool.')
-            SpicePool.add(kernels, purge=True)
+    @target.setter
+    @check_kernels
+    def target(self, target):
+        """Set target as a SpiceBody."""
+        log_traj.debug('Set Spice Target.')
+        self.__target = SpiceBody(target)
 
     @property
     def observer(self):
-        """Get observer."""
+        """Trajectory observer."""
         return self.__observer
 
     @observer.setter
+    @check_kernels
     def observer(self, observer):
+        """Set observer as a SpiceSpace or SpiceInstrument."""
+        log_traj.debug('Set Spice Observer.')
         self.__observer = SpiceRef(observer)
 
         if isinstance(self.observer, SpiceSpacecraft):
@@ -164,32 +170,41 @@ class Trajectory:  # pylint: disable=too-many-public-methods
 
     @property
     def spacecraft(self):
-        """Observer spacecraft SPICe reference."""
+        """Observer spacecraft SPICE reference."""
         return self.observer.spacecraft
 
     @property
     def ets(self):
-        """Ephemeris times."""
+        """Ephemeris Times (ET)."""
         return self.__ets
 
     @ets.setter
+    @check_kernels
     def ets(self, ets):
-        """Ephemeris times setter."""
-        log_traj.debug('Init ephemeris time.')
+        """Set Ephemeris Times (ET) array.
 
-        if isinstance(ets, str):
+        See Also
+        --------
+        TourConfig._parse for details.
+
+        """
+        log_traj.debug('Set ephemeris times array.')
+
+        if isinstance(ets, (str, int, float, np.datetime64)):
             self.__ets = np.array([et(ets)])
-        elif isinstance(ets, (int, float)):
-            self.__ets = np.array([ets])
-        elif isinstance(ets, slice):
-            self.__ets = et_range(ets.start, ets.stop, ets.step)
-        elif isinstance(ets, (tuple, list, np.ndarray)):
-            self.__ets = np.array(sorted([
-                et(t) if isinstance(t, str) else t for t in ets]))
+
+        elif isinstance(ets, (tuple, list, slice, np.ndarray)):
+            self.__ets = np.sort(et(ets))
+
         else:
             raise ValueError('Invalid input Ephemeris Time(s).')
 
-        if len(self.__ets) > self.TOO_MANY_POINTS_WARNING:
+        if isinstance(self.exclude, (EventWindow, EventsList)):
+            cond = self.exclude.contains(self.utc)
+            self.__ets = self.__ets[~cond]
+            del self.utc
+
+        if len(self.ets) > self.TOO_MANY_POINTS_WARNING:
             log_traj.warning(
                 'You have selected more than %s points. '
                 'SPICE computation can take a while. '
@@ -197,37 +212,28 @@ class Trajectory:  # pylint: disable=too-many-public-methods
                 'resolution or the time range.',
                 f'{self.TOO_MANY_POINTS_WARNING:,d}')
 
+        # Clear all cached properties (added by `@cached_property` decorators)
+        self.clear_cache()  # pylint: disable=no-member
+
     @cached_property
+    @check_kernels
     def utc(self):
         """UTC times."""
         log_traj.info('Compute UTC times.')
         return utc(self.ets)
 
-    @property
+    @cached_property
+    @check_kernels
     def start(self):
         """UTC start time."""
         return utc(self.ets[0])
 
-    @property
+    @cached_property
+    @check_kernels
     def stop(self):
         """UTC stop time."""
         return utc(self.ets[-1])
 
-    @property
-    def abcorr(self):
-        """SPICE aberration correction."""
-        return self.__abcorr
-
-    @abcorr.setter
-    def abcorr(self, key):
-        """SPICE aberration correction setter."""
-        if key.replace(' ', '').upper() not in [
-            'NONE', 'LT', 'LT+S', 'CN', 'CN+S',
-            'XLT', 'XLT+S', 'XCN', 'XCN+S',
-        ]:
-            raise KeyError(f'Invalid aberration correction key:`{key}`')
-        self.__abcorr = key.upper()
-
     @property
     def boresight(self):
         """Observer boresight pointing vector."""
@@ -257,6 +263,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
             'ECLIPJ2000'
 
     @cached_property
+    @check_kernels
     def sc_pts(self):
         """Sub-spacecraft surface intersection vector.
 
@@ -323,6 +330,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """Groundtrack or surface intersect (not implemented here)."""
 
     @cached_property
+    @check_kernels
     def boresight_pts(self):
         """Boresight surface intersection vector.
 
@@ -381,55 +389,8 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """
         return ~self.surface
 
-    @cached_property(parent='sc_pts')
-    def sc_illum_angles(self):
-        """Sub-spacecraft illumination angles (degree).
-
-        Incidence, emission and phase angles.
-
-        See Also
-        --------
-        Trajectory.inc
-        Trajectory.emi
-        Trajectory.phase
-        Trajectory.sc_pts
-
-        """
-        log_traj.info('Compute sub-spacecraft illumination geometry.')
-        return illum_angles(
-            self.ets,
-            self.observer,
-            self.target,
-            self.sc_pts,
-            abcorr=self.abcorr,
-            method='ELLIPSOID',
-        )
-
-    @cached_property(parent='boresight_pts')
-    def boresight_illum_angles(self):
-        """Instrument surface intersec illumination angles (degree).
-
-        Incidence, emission and phase angles.
-
-        See Also
-        --------
-        Trajectory.inc
-        Trajectory.emi
-        Trajectory.phase
-        Trajectory.boresight_pts
-
-        """
-        log_traj.info('Compute boresight intersection illumination geometry.')
-        return illum_angles(
-            self.ets,
-            self.observer,
-            self.target,
-            self.boresight_pts,
-            abcorr=self.abcorr,
-            method='ELLIPSOID',
-        )
-
     @cached_property
+    @check_kernels
     def sc_state(self):
         """Spacecraft position and velocity in the target body fixed frame (km and km/s).
 
@@ -516,7 +477,22 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         log_traj.debug('Compute spacecraft distance in the target frame.')
         return norm(self.sc_pos.T - self.sc_pts.T)
 
+    @cached_property(parent='dist')
+    def ets_target_center(self):
+        """Ephemeris Time (ET) at the target center.
+
+        The light time correction is only apply if :py:attr:`abcorr` in not ``NONE``.
+
+        See Also
+        --------
+        Trajectory.dist
+        SpiceAbCorr.dist_corr
+
+        """
+        return self.abcorr.dist_corr(self.ets, self.dist)
+
     @cached_property(parent='sc_state')
+    @check_kernels
     def groundtrack_velocity(self):
         """Compute the groundtrack velocity (km/s).
 
@@ -537,6 +513,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         return res
 
     @cached_property
+    @check_kernels
     def sc_attitude(self):
         """Spacecraft attitude c-matrix in J2000 frame."""
         log_traj.info('Compute spacecraft attitude.')
@@ -561,15 +538,14 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """Boresight pointing declination angle (degree)."""
         return self.radec[1]
 
-    @cached_property
+    @cached_property(parent='ets_target_center')
+    @check_kernels
     def sun_pos(self):
         """Sun position in the target body fixed frame (km).
 
         Note
         ----
-        Currently the intersection method is fixed on a sphere
-        and the input time at the target center is not corrected
-        from light travel.
+        Currently the intersection method is fixed on a sphere.
 
         See Also
         --------
@@ -582,7 +558,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """
         log_traj.info('Compute Sun coordinates in the target frame.')
         res = sun_pos(
-            self.ets,
+            self.ets_target_center,
             self.target,
             abcorr=self.abcorr)
 
@@ -630,7 +606,8 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """Alias on the sub-solar point groundtrack (degree)."""
         return self.sun_lonlat
 
-    @cached_property
+    @cached_property(parent='ets_target_center')
+    @check_kernels
     def solar_longitude(self):
         """Target seasonal solar longitude (degrees).
 
@@ -643,20 +620,21 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         """
         log_traj.info('Compute the target seasonal solar longitude.')
         res = solar_longitude(
-            self.ets,
+            self.ets_target_center,
             self.target,
             abcorr=self.abcorr)
 
         log_traj.debug('Result: %r', res)
         return res
 
-    @cached_property
+    @cached_property(parent='ets_target_center')
+    @check_kernels
     def true_anomaly(self):
         """Target orbital true anomaly (degree)."""
         log_traj.debug('Compute true anomaly angle.')
 
         res = true_anomaly(
-            self.ets,
+            self.ets_target_center,
             self.target,
             abcorr=self.abcorr,
             frame=self.inertial_frame)
@@ -665,8 +643,11 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         return res
 
     def mask(self, cond):
-        """Create a masked trajectory."""
-        return MaskedTrajectory(self, cond)
+        """Create a masked trajectory only where the condition invalid."""
+        if isinstance(self, InstrumentTrajectory):
+            return MaskedInstrumentTrajectory(self, cond)
+
+        return MaskedSpacecraftTrajectory(self, cond)
 
     def where(self, cond):
         """Create a masked trajectory only where the condition is valid."""
@@ -684,7 +665,7 @@ class Trajectory:  # pylint: disable=too-many-public-methods
 
         Returns
         -------
-        MaskedTrajectory
+        numpy.ndarray
             Mask to apply on the trajectory.
 
         Raises
@@ -697,32 +678,61 @@ class Trajectory:  # pylint: disable=too-many-public-methods
         if not hasattr(obj, 'contains'):
             raise AttributeError(f'Undefined `contains` intersection in {obj}.')
 
-        cond = obj.contains(self.lonlat)
+        cond = obj.contains(self)
         return cond if outside else ~cond
 
-    @property
-    def approx_ca_utc(self):
+    def approx_ca_utc(self, alt_min):
         """List of approximate closest approach UTC times to the target.
 
+        Parameters
+        ----------
+        alt_min: float
+            Minimal altitude at closest approach [km].
+
         Danger
         ------
         These values may not be accurate depending on the
         :py:class:`Trajectory` temporal resolution.
 
         """
+        dist_min = self.target.radius + alt_min
         return [
             self.utc[seg][np.argmin(self.dist[seg])]
             # pylint: disable=comparison-with-callable
-            for seg in self.where(self.dist <= Flyby.DIST_MIN).seg
+            for seg in self.where(self.dist <= dist_min).seg
+        ]
+
+    def get_flybys(self, alt_min=25_000):
+        """List of all the flybys on the target below a given altitude.
+
+        Parameters
+        ----------
+        alt_min: float, optional
+            Minimal altitude at closest approach (default: 25,000 km).
+
+        Returns
+        -------
+        [Flyby, …]
+            List of flybys below the required altitude.
+
+        """
+        return [
+            Flyby(self.kernels, self.observer, self.target, ca_utc,
+                  abcorr=self.abcorr, exclude=self.exclude,
+                  alt_min=alt_min)
+            for ca_utc in self.approx_ca_utc(alt_min)
         ]
 
     @property
     def flybys(self):
-        """Get all the flybys for this trajectory."""
-        return [
-            Flyby(self.kernels, self.observer, self.target, ca_utc, abcorr=self.abcorr)
-            for ca_utc in self.approx_ca_utc
-        ]
+        """Get all the flybys for this trajectory below 25,000 km.
+
+        See Also
+        --------
+        :py:func:`get_flybys` if you need a different minimal altitude.
+
+        """
+        return self.get_flybys()
 
 
 class SpacecraftTrajectory(Trajectory):
@@ -743,18 +753,6 @@ class SpacecraftTrajectory(Trajectory):
         """Sub-spacecraft groundtrack east longitudes and latitudes (degree)."""
         return self.sc_rlonlat[1:]
 
-    @cached_property(parent='sc_rlonlat')
-    def local_time(self):
-        """Sub-spacecraft local time (decimal hours).
-
-        See Also
-        --------
-        Trajectory.rlonlat
-
-        """
-        log_traj.info('Compute sub-spacecraft local time.')
-        return local_time(self.ets, self.lon_e, self.target)
-
     @property
     def slant(self):
         """Spacecraft line-of-sight distance to the sub-spacecraft point (km).
@@ -770,37 +768,89 @@ class SpacecraftTrajectory(Trajectory):
         """
         return self.alt
 
+    @cached_property(parent='alt')
+    def ets_surface_intersect(self):
+        """Ephemeris Time (ET) at the surface intersect point.
+
+        The light time correction is only apply if :py:attr:`abcorr` in not ``NONE``.
+
+        See Also
+        --------
+        Trajectory.alt
+        SpiceAbCorr.dist_corr
+
+        """
+        return self.abcorr.dist_corr(self.ets, self.alt)
+
+    @cached_property(parent=('sc_rlonlat', 'ets_surface_intersect'))
+    @check_kernels
+    def local_time(self):
+        """Sub-spacecraft local time (decimal hours).
+
+        See Also
+        --------
+        Trajectory.rlonlat
+
+        """
+        log_traj.info('Compute sub-spacecraft local time.')
+        return local_time(self.ets_surface_intersect, self.lon_e, self.target)
+
+    @cached_property(parent=('sc_pts', 'ets_surface_intersect'))
+    @check_kernels
+    def illum_angles(self):
+        """Sub-spacecraft illumination angles (degree).
+
+        Incidence, emission and phase angles.
+
+        See Also
+        --------
+        Trajectory.inc
+        Trajectory.emi
+        Trajectory.phase
+        Trajectory.sc_pts
+
+        """
+        log_traj.info('Compute sub-spacecraft illumination geometry.')
+        return illum_angles(
+            self.ets_surface_intersect,
+            self.observer,
+            self.target,
+            self.sc_pts,
+            abcorr=self.abcorr,
+            method='ELLIPSOID',
+        )
+
     @property
     def inc(self):
         """Sub-spacecraft incidence angle (degree)."""
-        return self.sc_illum_angles[0]
+        return self.illum_angles[0]
 
     @property
     def emi(self):
         """Sub-spacecraft emission angle (degree)."""
-        return self.sc_illum_angles[1]
+        return self.illum_angles[1]
 
     @property
     def phase(self):
         """Sub-spacecraft phase angle (degree)."""
-        return self.sc_illum_angles[2]
+        return self.illum_angles[2]
 
-    @cached_property(parent='sc_illum_angles')
+    @cached_property(parent='illum_angles')
     def day(self):
         """Day side, sub-spacecraft incidence lower than 90°."""
         return self.inc <= 90
 
-    @cached_property(parent='sc_illum_angles')
+    @cached_property(parent='illum_angles')
     def night(self):
         """Night side, sub-spacecraft incidence larger than 90°."""
         return self.inc > 90
 
-    @cached_property(parent='sc_illum_angles')
+    @cached_property(parent='illum_angles')
     def nadir(self):
         """Nadir viewing, always True for the sub-spacecraft point."""
         return np.full_like(self.ets, True, dtype=bool)
 
-    @cached_property(parent='sc_illum_angles')
+    @cached_property(parent='illum_angles')
     def off_nadir(self):
         """Off-nadir viewing, always False for the sub-spacecraft point."""
         return np.full_like(self.ets, False, dtype=bool)
@@ -837,9 +887,10 @@ class SpacecraftTrajectory(Trajectory):
         Trajectory.ell_norm
 
         """
+        # pylint: disable=no-member
         log_traj.debug('Compute sub-observer solar zenith angle.')
         res = angle(self.sun_pos.T - self.sc_pts.T,
-                    self.ell_norm.T)  # pylint: disable=no-member
+                    self.ell_norm.T)
 
         log_traj.debug('Result: %r', res)
         return res
@@ -847,8 +898,6 @@ class SpacecraftTrajectory(Trajectory):
 
 class InstrumentTrajectory(Trajectory):
     """Instrument trajectory object."""
-    FOV_PTS = 25
-
     @property
     def lon_e(self):
         """Instrument surface intersect east longitude (degree)."""
@@ -864,49 +913,89 @@ class InstrumentTrajectory(Trajectory):
         """Instrument surface intersect east longitudes and latitudes (degree)."""
         return self.boresight_rlonlat[1:]
 
-    @cached_property(parent='boresight_rlonlat')
-    def local_time(self):
-        """Instrument surface intersec local time (decimal hours)."""
-        log_traj.info('Compute boresight intersection local time.')
-        return local_time(self.ets, self.lon_e, self.target)
-
     @cached_property(parent=('sc_pts', 'boresight_pts'))
     def slant(self):
         """Line-of-sight distance to the boresight surface intersection (km)."""
         log_traj.debug('Compute slant range to the boresight surface intersection.')
         return norm(self.sc_pos.T - self.boresight_pts.T)
 
+    @cached_property(parent='slant')
+    def ets_surface_intersect(self):
+        """Ephemeris Time (ET) at the surface intersect point.
+
+        The light time correction is only apply if :py:attr:`abcorr` in not ``NONE``.
+
+        See Also
+        --------
+        Trajectory.slant
+        SpiceAbCorr.dist_corr
+
+        """
+        return self.abcorr.dist_corr(self.ets, self.slant)
+
+    @cached_property(parent=('boresight_rlonlat', 'ets_surface_intersect'))
+    @check_kernels
+    def local_time(self):
+        """Instrument surface intersec local time (decimal hours)."""
+        log_traj.info('Compute boresight intersection local time.')
+        return local_time(self.ets_surface_intersect, self.lon_e, self.target)
+
+    @cached_property(parent=('boresight_pts', 'ets_surface_intersect'))
+    @check_kernels
+    def illum_angles(self):
+        """Instrument surface intersec illumination angles (degree).
+
+        Incidence, emission and phase angles.
+
+        See Also
+        --------
+        Trajectory.inc
+        Trajectory.emi
+        Trajectory.phase
+        Trajectory.boresight_pts
+
+        """
+        log_traj.info('Compute boresight intersection illumination geometry.')
+        return illum_angles(
+            self.ets_surface_intersect,
+            self.observer,
+            self.target,
+            self.boresight_pts,
+            abcorr=self.abcorr,
+            method='ELLIPSOID',
+        )
+
     @property
     def inc(self):
         """Instrument surface intersec incidence angle (degree)."""
-        return self.boresight_illum_angles[0]
+        return self.illum_angles[0]
 
     @property
     def emi(self):
         """Instrument surface intersec emission angle (degree)."""
-        return self.boresight_illum_angles[1]
+        return self.illum_angles[1]
 
     @property
     def phase(self):
         """Instrument surface intersec phase angle (degree)."""
-        return self.boresight_illum_angles[2]
+        return self.illum_angles[2]
 
-    @cached_property(parent='boresight_illum_angles')
+    @cached_property(parent='illum_angles')
     def day(self):
         """Day side, boresight intersection incidence lower than 90°."""
         return self.inc <= 90
 
-    @cached_property(parent='boresight_illum_angles')
+    @cached_property(parent='illum_angles')
     def night(self):
         """Night side, boresight intersection incidence larger than 90°."""
         return self.inc > 90
 
-    @cached_property(parent='boresight_illum_angles')
+    @cached_property(parent='illum_angles')
     def nadir(self):
         """Nadir viewing, boresight intersection emission angle is lower than 1°."""
         return self.emi < 1
 
-    @cached_property(parent='boresight_illum_angles')
+    @cached_property(parent='illum_angles')
     def off_nadir(self):
         """Off-nadir viewing, boresight intersection emission angle is larger than 1°."""
         return self.emi >= 1
@@ -943,82 +1032,18 @@ class InstrumentTrajectory(Trajectory):
         Trajectory.ell_norm
 
         """
+        # pylint: disable=no-member
         log_traj.debug('Compute sub-observer solar zenith angle.')
         res = angle(self.sun_pos.T - self.boresight_pts.T,
-                    self.ell_norm.T)  # pylint: disable=no-member
+                    self.ell_norm.T)
 
         log_traj.debug('Result: %r', res)
         return res
 
     @cached_property
-    def fov_pts(self):
-        """Instrument FOV points.
-
-        Note
-        ----
-        Currently the intersection method is fixed internally
-        as ``method='ELLIPSOID'``.
-
-        """
-        log_traj.debug('Compute FOV intersection points.')
-        res = fov_pts(
-            self.ets,
-            self.observer,
-            self.target,
-            limb=True,
-            npt=self.FOV_PTS - 1,
-            abcorr=self.abcorr,
-            method='ELLIPSOID')
-
-        # Close the polygon
-        res = np.dstack([res, res[..., 0]])
-
-        log_traj.debug('Result: %r', res)
-        return res
-
-    @cached_property(parent='fov_pts')
-    def fov_rlonlat(self):
-        """Instrument FOV intersect coordinates.
-
-        Returns
-        -------
-        np.ndarray
-            Boresight surface intersect planetocentric coordinates:
-            radii, east longitudes and latitudes.
-
-        See Also
-        --------
-        Trajectory.fov_pts
-
-        """
-        log_traj.debug('Compute FOV intersects planetocentric coordinates.')
-        return rlonlat(self.fov_pts)
-
-    @cached_property(parent='fov_rlonlat')
-    def fov_paths(self):
-        """Instrument FOV surface paths.
-
-        Note
-        ----
-        If all the points are above the limb the path is set to ``None``.
-
-        See Also
-        --------
-        Trajectory.fov_rlonlat
-
-        """
-        log_traj.debug('Compute FOV paths.')
-        r_max = max(self.target.radii)
-
-        return [
-            Path(rlonlat[..., 1:]) if np.min(rlonlat[..., 0]) < r_max else None
-            for rlonlat in np.moveaxis(self.fov_rlonlat, 0, -1)
-        ]
-
-    def fovs(self, label=None, **kwargs):
-        """Instrument field of view paths collection."""
-        label = str(self.observer).replace('_', ' ') if label is None else label
-        return PathCollection(self.fov_paths, label=label, **kwargs)
+    def fovs(self):
+        """Instrument field of views collection."""
+        return FovsCollection(self)
 
 
 class Flyby(Trajectory):
@@ -1036,7 +1061,7 @@ class Flyby(Trajectory):
     - 3rd stage with a fine resolution (1 sec at CA ± 2 min)
 
     By default the final sampling temporal steps are irregular
-    with a high resolution only around the CA:
+    with a high resolution only around CA:
 
     - 1 pt from CA -12 h to CA  -2 h every 10 min
     - 1 pt from CA  -2 h to CA  -1 h every  1 min
@@ -1074,78 +1099,113 @@ class Flyby(Trajectory):
         the target's position and orientation.
         Only the SPICE keys are accepted.
 
+    exclude: EventWindow or EventsList
+        Event window or list of events to exclude from the analysis.
+
+    alt_min: float, optional
+        Altitude minimal to a given target [km] (default: 25,000 km).
+
     See Also
     --------
     Trajectory
     moon_coverage.spice.et_ca_range
 
-    Raises
-    ------
-    AltitudeTooHighError
-        If the target altitude is too high (above ``100,000 km``)
-
     """
-
-    DIST_MIN = 100_000  # Distance minimal to a given target [km]
-
-    def __init__(self, kernels, observer, target, approx_ca_utc, *dt, abcorr='NONE'):
-        # Check the pool coverage
-        et_min, et_max = SpicePool.coverage(observer, target, fmt='ET')
-
-        # Coarse stage
-        coarse_ets = et_ca_range(approx_ca_utc, (24, 'h', '20 min'))
-        coarse_ets = coarse_ets[(et_min <= coarse_ets) & (coarse_ets <= et_max)]
-
-        log_traj.info('Coarse UTC window: %s', utc(coarse_ets[0], coarse_ets[-1]))
-
-        coarse_traj = Trajectory(kernels, observer, target, coarse_ets)
-        coarse_ca_utc = coarse_traj.utc[np.argmin(coarse_traj.dist)]
-
-        log_traj.info('Coarse CA UTC: %s', coarse_ca_utc)
-
-        # Medium stage
-        medium_ets = et_ca_range(coarse_ca_utc, (30, 'm', '1 min'))
-        medium_ets = medium_ets[(et_min <= medium_ets) & (medium_ets <= et_max)]
-
-        log_traj.info('Medium UTC window: %s', utc(medium_ets[0], medium_ets[-1]))
-
-        medium_traj = Trajectory(kernels, observer, target, medium_ets)
-        medium_ca_utc = medium_traj.utc[np.argmin(medium_traj.dist)]
-
-        log_traj.info('Medium CA UTC: %s', medium_ca_utc)
-
-        # Fine stage
-        fine_ets = et_ca_range(medium_ca_utc, (2, 'm', '1 sec'))
-        fine_ets = fine_ets[(et_min <= fine_ets) & (fine_ets <= et_max)]
-
-        log_traj.info('Fine UTC window: %s', utc(fine_ets[0], fine_ets[-1]))
-
-        fine_traj = Trajectory(kernels, observer, target, fine_ets)
-        fine_ca_utc = fine_traj.utc[np.argmin(fine_traj.alt)]
-
-        # Round [ms] digits to 1 sec precision
-        fine_ca_utc = (fine_ca_utc + np.timedelta64(500, 'ms')).astype('datetime64[s]')
-
-        log_traj.info('Fine CA UTC: %s', fine_ca_utc)
-
-        # Check min altitude to make sure its valid flyby
-        fine_ca_alt = fine_traj.alt[np.argmin(fine_traj.alt)]
-
-        if fine_ca_alt > self.DIST_MIN:
-            raise AltitudeTooHighError(
-                f'{fine_ca_alt:,.1f} km > {self.DIST_MIN:,.1f} km at CA')
-
-        # Select the flyby temporal grid
-        ets = et_ca_range(fine_ca_utc, *dt)
-        ets = ets[(et_min <= ets) & (ets <= et_max)]
-
-        super().__init__(kernels, observer, target, ets)
+    def __init__(self, kernels, observer, target, approx_ca_utc, *dt,
+                 abcorr='NONE', exclude=None, alt_min=25_000):
+        # Init with an empty temporal grid
+        super().__init__(kernels, observer, target, [], abcorr=abcorr, exclude=exclude)
 
         if isinstance(self, SpacecraftTrajectory):
             self.__class__ = SpacecraftFlyby
         else:
             self.__class__ = InstrumentFlyby
 
+        # Create the flyby temporal grid (symmetrical around CA)
+        self.ets = self._flyby_ca_ets(approx_ca_utc, alt_min, dt)
+
+    @check_kernels
+    def _flyby_ca_ets(self, approx_ca_utc, alt_min, dt):
+        """Closest Approach (CA) Ephemeris Times for a flyby.
+
+        Parameters
+        ----------
+        approx_ca_utc: float, string or numpy.datetime64
+            Approximate CA datetime, used as an initial step.
+
+        alt_min: float, optional
+            Altitude minimal to a given target [km] (default: 25,000 km).
+
+        dt: tuple(s), optional
+            Temporal sequence around closest approach:
+
+            `(duration, numpy.datetime unit, step value and unit)`
+
+            See :py:func:`moon_coverage.spice.et_ca_range` for more details.
+
+        Returns
+        -------
+        np.ndarray
+            Ephemeris times around CA.
+
+        Raises
+        ------
+        AltitudeTooHighError
+            If the target altitude at CA is too high (above ``alt_min``).
+
+        Note
+        ----
+        The light time correction (``abcorr``) is only apply in the lastest
+        search stage.
+
+        """
+        # Check the pool coverage
+        et_min, et_max = SpicePool.coverage(self.observer, self.target, fmt='ET')
+        ets_coverage = {'et_min': et_min, 'et_max': et_max}
+
+        # 1 - Coarse stage (20 min at CA ± 12h)
+        coarse_ets = et_ca_range(approx_ca_utc, (24, 'h', '20 min'), **ets_coverage)
+        log_traj.info('Coarse UTC window: %s', utc(coarse_ets[0], coarse_ets[-1]))
+
+        coarse_traj = Trajectory(self.kernels, self.observer, self.target, coarse_ets)
+        coarse_ca_utc = coarse_traj.utc[np.argmin(coarse_traj.dist)]
+        log_traj.info('Coarse CA UTC: %s', coarse_ca_utc)
+
+        # 2 - Medium stage (1 min at CA ± 30 min)
+        medium_ets = et_ca_range(coarse_ca_utc, (30, 'm', '1 min'), **ets_coverage)
+        log_traj.info('Medium UTC window: %s', utc(medium_ets[0], medium_ets[-1]))
+
+        medium_traj = Trajectory(self.kernels, self.observer, self.target, medium_ets)
+        medium_ca_utc = medium_traj.utc[np.argmin(medium_traj.dist)]
+        log_traj.info('Medium CA UTC: %s', medium_ca_utc)
+
+        # 3 - Fine stage (1 sec at CA ± 2 min)
+        fine_ets = et_ca_range(medium_ca_utc, (2, 'm', '1 sec'), **ets_coverage)
+        log_traj.info('Fine UTC window: %s', utc(fine_ets[0], fine_ets[-1]))
+
+        fine_traj = Trajectory(self.kernels, self.observer, self.target, fine_ets,
+                               abcorr=self.abcorr, exclude=self.exclude)
+
+        # Locate CA as min `alt` (not `dist` this time)
+        i_ca = np.argmin(fine_traj.alt)
+
+        # Check min altitude to make sure its valid flyby
+        if fine_traj.alt[i_ca] > alt_min:
+            raise AltitudeTooHighError(
+                f'{fine_traj.alt[i_ca]:,.1f} km > {alt_min:,.1f} km at CA'
+            )
+
+        # Round [ms] digits to 1 sec precision
+        fine_ca_utc = (
+            fine_traj.utc[i_ca] + np.timedelta64(500, 'ms')
+        ).astype('datetime64[s]')
+        log_traj.info('Fine CA UTC: %s', fine_ca_utc)
+
+        # Compute the point around CA with `dt` time steps
+        ets = et_ca_range(fine_ca_utc, *dt, **ets_coverage)
+
+        return ets
+
     def __repr__(self):
         return (
             f'<{self.__class__.__name__}> '
@@ -1213,6 +1273,21 @@ class Flyby(Trajectory):
         """Altitude at closest approach (km)."""
         return self.alt[self._i_ca]
 
+    @property
+    def inc_ca(self):
+        """Incidence angle at closest approach (degree)."""
+        return self.inc[self._i_ca]  # pylint: disable=no-member
+
+    @property
+    def emi_ca(self):
+        """Emission angle at closest approach (degree)."""
+        return self.emi[self._i_ca]  # pylint: disable=no-member
+
+    @property
+    def phase_ca(self):
+        """Phase angle at closest approach (degree)."""
+        return self.phase[self._i_ca]  # pylint: disable=no-member
+
     @cached_property
     def ca(self):
         """Closest approach point."""
